unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, EditPlugAPI, FTPSend, inifiles, uEncrypt, ComCtrls;

type
  TForm1 = class(TForm)
    btnFTPUpload: TButton;
    edHost: TEdit;
    edPort: TEdit;
    edUser: TEdit;
    edPw: TEdit;
    lblPort: TLabel;
    lblUser: TLabel;
    lblPw: TLabel;
    lblHost: TLabel;
    cbDesc: TComboBox;
    lblDesc: TLabel;
    edSaveAs: TEdit;
    lblSaveAs: TLabel;
    btnSaveConnection: TButton;
    edSubdir: TEdit;
    lblSubdir: TLabel;
    edDesc: TEdit;
    ProgressBar1: TProgressBar;
    lblProgress: TLabel;
    lblFile: TLabel;
    edFile: TEdit;
    Button1: TButton;
    procedure btnFTPUploadClick(Sender: TObject);
    procedure btnSaveConnectionClick(Sender: TObject);
    procedure cbDescChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OnShowFirst;
    procedure UpdateCbDesc;
    procedure UpdateFtpFormInfo(item: string);
    procedure SelectDescCB(item: string);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
  private
    FirstShow: boolean;
    { Private declarations }
  public
    { Public declarations }
  end;

const
  DEFAULT_KEY = 10453;

var
  Form1: TForm1;
  gPlugId: HINST;

implementation

{$R *.DFM}

procedure msg(s: string);
begin
  showmessage(s);
end;

{----- EXPORTS ----------------------------------------------------------------}

procedure plgSetId(id: HINST); cdecl;
begin
  gPlugId:= id;
end; exports plgSetId;

procedure plgExceptTest1; cdecl;
var x: integer;
begin
  x:= strtoint('abcdefg');
  showmessage(inttostr(x));
end; exports plgExceptTest1;

function plgShow: HWND; cdecl;
begin
  if not assigned(form1) then begin
    form1:= tform1.create(nil);
    epSetParentToMainWin(form1.handle);
    form1.show;
    epMoveWinToTopRight(form1);
  end else begin
    form1.show;
  end;
  result:= form1.Handle;
end; exports plgShow;

procedure plgWinFree; cdecl;
begin
  if assigned(form1) then begin
    form1.Free;
    form1:=nil;
  end;
end; exports plgWinFree;

function plgGetWinHandle: HWND; cdecl;
begin
  result:=0;
  if assigned(form1) then result:= form1.Handle;
end; exports plgGetWinHandle;


{------------------------------------------------------------------------------}


const TestFName = 'c:\delete\tmp\test2.txt';

function FieldsBlank: boolean;
begin
  result := false;
  if form1.edHost.Text = '' then result := true;
  if form1.edPort.Text = '' then result := true;
  if form1.edUser.Text = '' then result := true;
  if form1.edFile.Text = '' then result := true;
end;

function CorrectSubdirSlash(s: string): string;
var len: integer;
begin
  result := '';
  if form1.edSubdir.text = '' then exit;
  len := length(s);
  if form1.edSubdir.text[len] = '/' then
    result := s // file path is correct
  else
    result := s + '/'; // file path needs trailing slash

end;

procedure ShowLibraryPath;
var
  Buffer: Array[0..260] of Char;
begin
  GetModuleFileName(hInstance, Buffer, Length(Buffer));
  ShowMessage(Buffer);
end;

function LibPath: string;
var
  Buffer: Array[0..260] of Char;
begin
  GetModuleFileName(hInstance, Buffer, Length(Buffer));
  result := buffer;
end;

function FtpLastInfoPath: string;
begin
//  result := ExtractFilePath(paramstr(0)) + 'plugins\FTPSave\' + 'lastftp.ini';
  result := ExtractFilePath(LibPath) + 'lastftp.ini';
end;

function FtpIniPath: string;
begin
  result := ExtractFilePath(paramstr(0)) + 'plugins\FTPSave\' + 'ftp.ini';
end;

const  LASTFTP_SECTION = 'Last Ftp';

procedure SaveLastFtpInfo;
var ini: TIniFile;
begin
  Ini := TIniFile.Create(FtpLastInfoPath);
  try
    Ini.WriteString(LASTFTP_SECTION, 'description', form1.cbDesc.text);
    Ini.WriteString(LASTFTP_SECTION, 'subdir', form1.edSubdir.text);
  finally
    Ini.Free;
  end;
end;

procedure TForm1.SelectDescCB(item: string);
var i: integer;
begin
  if cbDesc.Items.Count < 1 then exit;
  for i := 0 to cbDesc.Items.Count - 1 do begin
    if cbDesc.items[i] = item then begin
      cbDesc.ItemIndex := i;
      edDesc.text := cbDesc.text;
    end;
  end;
end;

procedure LoadLastFtpInfo;
var
  ini: TIniFile;
  sdesc, ssubdir: string;
begin
  if not fileexists(FtpLastInfoPath) then exit;
  Ini := TIniFile.Create(FtpLastInfoPath);
  try
    sdesc := Ini.ReadString(LASTFTP_SECTION, 'description', '');
    if sdesc <> '' then begin
      form1.SelectDescCB(sdesc);
    end;
    ssubdir := Ini.ReadString(LASTFTP_SECTION, 'subdir', '');
    form1.edSubdir.text := ssubdir;
  finally
    Ini.Free;
  end;
end;

procedure TForm1.btnFTPUploadClick(Sender: TObject);
var
  FFTP: TFTPSend;
  subdir: string;

  procedure LogoutAndFree;
  begin
    FFTP.Logout;
    FFTP.free; FFTP := nil;
    ProgressBar1.Position := 0;
  end;

begin
  if FieldsBlank then begin
    msg('A required field was left blank. Cannot upload.');
    exit;
  end;

  FFTP := TFTPSend.Create;

  FFTP.TargetHost := edHost.text;
  FFTP.TargetPort := edPort.text;

  FFTP.Username := edUser.text; //'test1@z505.com';
  FFTP.Password := edPw.text; //'test1pass';

  FFTP.Timeout := 10000;
  FFTP.PassiveMode := true;
  if not FFTP.Login then begin
    msg('could not log in with login details provided');
    LogoutAndFree;
    exit;
  end else begin
  //  FFTP.FWMode := ;
  //  FFTP.FWHost := '';
  //  FFTP.FWPort := ;
  //  FFTP.FWUsername := '';
  //  FFTP.FWPassword := '';

    FFTP.DirectFile := true;
    FFTP.DirectFileName := edFile.text;

    subdir := CorrectSubdirSlash(edSubdir.text);
    ProgressBar1.Position:=10;
    if FFTP.StoreFile(Subdir + edSaveAs.text, false) then begin
      ProgressBar1.Position := 100;
//      showmessage('file uploaded');
      SaveLastFtpInfo;
    end else begin
      showmessage('problem uploading, check file path');
      LogoutAndFree;
      exit;
    end;
  end;
  LogoutAndFree;
  //  self.close;
  self.Release;
  form1 := nil;
end;

procedure TForm1.UpdateFtpFormInfo(item: string);
begin
  UpdateCbDesc;
  selectdesccb(item);
end;

procedure TForm1.btnSaveConnectionClick(Sender: TObject);
var ini: TIniFile;
begin
  if edDesc.text = '' then begin
    msg('Description field cannot be blank');
    exit;
  end;
  Ini := TIniFile.Create(FtpIniPath);
  try
    Ini.WriteString(edDesc.Text, 'host', edHost.text);
    Ini.WriteString(edDesc.Text, 'port', edPort.text);
    Ini.WriteString(edDesc.Text, 'user', edUser.text);
    Ini.WriteString(edDesc.Text, 'pw', Encrypt(edPw.text, DEFAULT_KEY));
  finally
    Ini.Free;
  end;
  UpdateFtpFormInfo(edDesc.Text);
end;

procedure TForm1.cbDescChange(Sender: TObject);
var
  ini: TIniFile;
begin
  if cbDesc.Text = '' then exit;
  Ini := TIniFile.Create(FtpIniPath);
  try
    edHost.text := Ini.ReadString(cbDesc.Text, 'host', '');
    edPort.text := Ini.ReadString(cbDesc.Text, 'port', '');
    edUser.text := Ini.ReadString(cbDesc.Text, 'user', '');
    edPw.text := decrypt(Ini.ReadString(cbDesc.Text, 'pw', ''), DEFAULT_KEY);
    edDesc.Text := cbDesc.Text;
  finally
    Ini.Free;
  end;
end;

procedure TForm1.FormShow(Sender: TObject);
const counter: integer = 0;
begin
  if FirstShow = true then begin
    FirstShow := false;
    OnShowFirst;
  end;
end;

procedure TForm1.UpdateCbDesc;
var
  ini: TIniFile;
begin
  Ini := TIniFile.Create(FtpIniPath);
  try
    Ini.ReadSections(cbDesc.Items);
  finally
    Ini.Free;
  end;
end;

procedure TForm1.OnShowFirst;
var
  sections: TStringList;
  ini: TIniFile;
begin
  if not FileExists(FtpIniPath) then exit;
  UpdateCbDesc;
  if cbDesc.Items.Count > 0 then begin
    LoadLastFtpInfo;
    edDesc.Text := cbDesc.Text;
    cbDescChange(nil);
  end;
  //showmessage('filename as string: ' + epGetActiveEdFilenameAsString);
  edFile.text := epGetActiveEdFilenameAsString;
  edSaveAs.Text := ExtractFileName(edFile.text);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FirstShow := true;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
  form1 := nil;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  showmessage(FtpLastInfoPath);
end;

end.
